unit ucuCadastrarLivro;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework, nguLivro;

type
  ucCadastrarLivro = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  private
    fLivro: ngLivro;
  published
    procedure Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ConcluirEdicao(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_Cancelar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage {define utField},
  utuEmprestimoOQL,
  acuObject {define acEnumerator},
  nguExemplar,

  nguSituacaoExemplar,

  nguEmprestimo,

  nguCliente,

  dateUtils;

procedure ucCadastrarLivro.Initialize;
begin
  inherited;
end;

procedure ucCadastrarLivro.Finalize;
begin
  
  inherited;
end;

procedure ucCadastrarLivro.Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivros,
  lFieldLivro: utField;
  lOQL: IObterLivros;
  lListLivros: acPersistentObjectList;
  lEnumLivro: acEnumerator;
  lLivro: ngLivro;
begin
  lFieldLivro:= piRequest.Message.RootField.FieldByName('Livro');

  lOQL := utEmprestimoOQL.ObterLivros(Self.Session);

  lFieldLivros := piRequest.Response.RootField.AddField('Livros');

  lListLivros := acPersistentObjectList.Create(ngLivro);
  try

    if lFieldLivro.HasAttribute('Titulo')
      then lOQL.Param_titulo.Value := lFieldLivro.AttributeByName('Titulo').AsString
      else lOQL.Param_titulo.SetNull;

    if lFieldLivro.HasAttribute('Autor')
      then lOQL.Param_autor.Value := lFieldLivro.AttributeByName('Autor').AsString
      else lOQL.Param_autor.SetNull;

    lOQL.LoadList(lListLivros);

    lEnumLivro := acEnumerator.Create(lListLivros);
    try
      while not lEnumLivro.EOL do
      begin
        lLivro := lEnumLivro.Current as ngLivro;
        lLivro.toXML(lFieldLivros.AddField('Livro'));

        lEnumLivro.MoveNext;
      end;
    finally
      lEnumLivro.Free;
    end;


  finally
    lListLivros.Free;
  end;

  piRequest.Processed := True;
end;

{Devido ao modelo, s ocorrer a excluso se o livro no possuir emprstimos.}
{Se ele possuir exemplares, eles sero deletados juntos com o livro.}
procedure ucCadastrarLivro.Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro: utField;
  lLivro: ngLivro;
  lListExemplares: acPersistentObjectList;
begin

  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');
  try
    lLivro := Self.Session.LoadObject(ngLivro,acOIDInt.Create(lFieldLivro.AttributeByName('OID').AsInteger)) as ngLivro;
    Self.Session.DeleteObject(lLivro,ngLivro);
  except
    on e:acObjectNotFoundException do
      raise EucException.Create('Livro a ser deletado no encontrado.');
  else
      raise EucException.Create('No foi possvel buscar o Livro a ser deletado.');
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ConcluirEdicao(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_Cancelar(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3227,ucCadastrarLivro);

end.
